#include <stdlib.h>
#include "ZUnpack.h"
#include "W3RepAPI.h"

TW3RepFile::TW3RepFile()
{
    m_loaded = false;
    m_playercount = 0;
    m_slotcount = 0;
    m_startspotcount = 0;
    for(int i = 0; i < 12; i ++)
        m_playerindex[i] = 0xFF;
}

TW3RepFile::~TW3RepFile()
{
    Unload();
}

void ProcessActionCB(TW3RepFile * W3RepFile, TW3RepAction * action, bool add)
{
    if(add)
        W3RepFile->Player[W3RepFile->GetPlayerIndexByID(action->GetPlayerID())].IncrAction();
    if(action->GetID() == aid_leavegame)
    {
        W3RepFile->Player[W3RepFile->GetPlayerIndexByID(action->GetPlayerID())].SetPlayTime(action->GetTime());
    }
}

bool TW3RepFile::Load(const char * FileName, WRL_Byte options)
{
    FILE * fp;
    WRL_String data = NULL, p_data;
    WRL_DWord data_size;
    bool ladder;
    unsigned int i;

/*
    Open file
*/
    if((!FileName) || FileName[0] == 0)
        return false;
    fp = fopen(FileName, "rb");
	if(fp == NULL)
        return false;

    if(m_loaded)
        Unload();

/*
    Read header
*/
    if(!Header.Load(fp))
    {
        fclose(fp);
        return false;
    }

/*
    Decompress blocks
*/
    if(!zunpack_data(fp, Header.Header.block_num, data, data_size))
    {
        fclose(fp);
        return false;
    }
    fclose(fp);

    fp = fopen("D:\\1.dat", "w+");
    fwrite(data, data_size, 1, fp);
    fclose(fp);

/*
    Read data
*/
/*
    Read host player data
*/
    p_data = data + 4;
    Player[0].Load1(p_data, &ladder);
    if(ladder)
        GameData.SetGameType(gametype_ladder);
    else
        GameData.SetGameType(gametype_custom);

/*
    Read Game Data
*/

    GameData.Load(p_data);

/*
    Read Players Info
*/
    i = 1;
    while(Player[i].Load1(p_data, NULL)) i++;
    m_playercount = i;

    for(i = 0; i < 12; i ++)
        Player[i].SetPlayTime(Header.GetLength());
    for(i = 0; i < 12; i ++)
    {
        if(Player[i].GetID() > 0 && Player[i].GetID() < 13)
            m_playerindex[Player[i].GetID() - 1] = i;
    }

    if((*p_data) == 0x19)
    {
        int flag;
        p_data += 3;
        m_slotcount = *p_data;
        p_data ++;
        for(i = 0; i < m_slotcount; i ++)
        {
            int j;
            if(*p_data > 0 && (j = GetPlayerIndexByID(*p_data)) != 0xff)
                Player[j].Load2(p_data, Header.GetVersion());
            else
                for(j = 0; j < 12; j ++)
                    if(!Player[j].Loaded())
                    {
                        Player[j].Load2(p_data, Header.GetVersion());
                        if(Player[j].GetSlotStatus() == slotstatus_used)
                            m_playercount ++;
                        break;
                    }
        }
        /* Skip Random Seed */
        p_data += 4;
        /* Read Map settings in WE */
        flag = GameData.GetGameFlag();
        switch(*p_data)
        {
        case 1: flag &= gameflag_fixedally; break;
        case 3: flag &= gameflag_fixedplayerprop; break;
        case 4: flag &= gameflag_fixracetorandom; break;
        }
        GameData.SetGameFlag(flag);
        p_data ++;
        m_startspotcount = *p_data;
        p_data ++;
    }
    if(options & COUNT_ACTIONS)
    {
        ActionList.SetProcessActionCallBack(ProcessActionCB);
        ActionList.Load(this, p_data, (options & ACTIONS_DETAIL) > 0, (options & EXTRA_ACTIONS) > 0, Header.GetBuild());
    }

    free((void *)data);

    m_loaded = true;
	return true;
}

void TW3RepFile::Unload()
{
    int i;
    if(!m_loaded)
        return;
    m_loaded = false;

    Header.Unload();
    GameData.Unload();
    ActionList.Unload();
    for(i = 0; i < 12; i ++)
        Player[i].Unload();
    m_playercount = 0;
    m_slotcount = 0;
    m_startspotcount = 0;
    for(i = 0; i < 12; i ++)
        m_playerindex[i] = 0xFF;
}

bool TW3RepFile::Loaded()
{
    return m_loaded;
}

WRL_Byte TW3RepFile::GetPlayerCount()
{
    return m_playercount;
}

WRL_Byte TW3RepFile::GetSlotCount()
{
    return m_slotcount;
}

WRL_Byte TW3RepFile::GetStartSpotCount()
{
    return m_startspotcount;
}

WRL_Byte TW3RepFile::GetPlayerIndexByID(WRL_Byte playerid)
{
    return m_playerindex[playerid - 1];
}
