#ifndef W3RepActionH
#define W3RepActionH

#include "W3RepDataType.h"

class TW3RepFile;

enum {
    COUNT_ACTIONS = 1,
    ACTIONS_DETAIL = 2,
    EXTRA_ACTIONS = 4
};

typedef enum {
    aid_unknown = 0x00,
    aid_pause = 0x01,
    aid_resume = 0x02,
    aid_setspeed = 0x03,
    aid_incrspeed = 0x04,
    aid_decrspeed = 0x05,
    aid_savegame = 0x06,
    aid_savefinish = 0x07,
    aid_ability_noparam = 0x10,
    aid_ability_pos = 0x11,
    aid_ability_posid = 0x12,
    aid_giveitem = 0x13,
    aid_ability_2posid = 0x14,
    aid_selection = 0x16,
    aid_assign = 0x17,
    aid_hotkey = 0x18,
    aid_subgroup = 0x19,
    aid_subgroup2 = 0x1A,
    aid_unknown1A = 0x1A,
    aid_unknown1B = 0x1B,
    aid_selectitem = 0x1B,
    aid_selectitem2 = 0x1C,
    aid_cancelrevive = 0x1C,
    aid_cancelrevive2 = 0x1D,
    aid_removequeue = 0x1D,
    aid_removequeue2 = 0x1E,
    aid_unknown21 = 0x21,
    aid_cheat1 = 0x20,
    aid_cheat2 = 0x22,
    aid_cheat3 = 0x23,
    aid_cheat4 = 0x24,
    aid_cheat5 = 0x25,
    aid_cheat6 = 0x26,
    aid_cheat7 = 0x27,
    aid_cheat8 = 0x28,
    aid_cheat9 = 0x29,
    aid_cheat10 = 0x2A,
    aid_cheat11 = 0x2B,
    aid_cheat12 = 0x2C,
    aid_cheat13 = 0x2D,
    aid_cheat14 = 0x2E,
    aid_cheat15 = 0x2F,
    aid_cheat16 = 0x30,
    aid_cheat17 = 0x31,
    aid_cheat18 = 0x32,
    aid_changeally = 0x50,
    aid_transfer = 0x51,
    aid_trigger = 0x60,
    aid_escpress = 0x61,
    aid_scentrigger = 0x62,
    aid_chooseskill = 0x65,
    aid_chooseskill2 = 0x66,
    aid_choosebuild = 0x66,
    aid_choosebuild2 = 0x67,
    aid_signal = 0x67,
    aid_signal2 = 0x68,
    aid_continueB = 0x68,
    aid_continueB2 = 0x69,
    aid_continueA = 0x69,
    aid_continueA2 = 0x6A,
    aid_unknown75 = 0x75,
    aid_chat = 0x40,
    aid_leavegame = 0x41
} TW3RepActionID;

typedef enum {
    ParamType_WRL_Byte,
    ParamType_WRL_Word,
    ParamType_WRL_DWord,
    ParamType_UnitID,
    ParamType_WRL_Float,
    ParamType_Pos,
    ParamType_WRL_String
} TW3RepActionParamType;

typedef struct {
    TW3RepActionParamType type;
    void * data;
} TW3RepActionParam;

class TW3RepAction;

typedef const WRL_String (*TParamFunc)(const TW3RepAction * action);

class TW3RepAction
{
public:
    TW3RepAction(TW3RepFile * owner = 0, TParamFunc paramfunc = 0):m_id(aid_unknown), m_playerid(0), m_time(0), m_params(0), m_paramcount(0), m_owner(owner), m_paramfunc(paramfunc) {}
    virtual ~TW3RepAction();
private:
    TW3RepFile * m_owner;
    TW3RepActionID m_id;
    WRL_Byte m_playerid;
    WRL_DWord m_time;
    TW3RepActionParam * m_params;
    WRL_Byte m_paramcount;
    TParamFunc m_paramfunc;
public:
    void Initial(TW3RepFile * owner, TParamFunc paramfunc);
    void FreeParams();
    TW3RepFile * GetOwner() const;
    TW3RepActionID GetID() const;
    WRL_String GetName() const;
    WRL_Byte GetPlayerID() const;
    WRL_DWord GetTime() const;
    void SetID(TW3RepActionID aid);
    bool SetPlayerID(WRL_Byte playerid);
    void SetTime(WRL_DWord time);
    void AddParam(TW3RepActionParamType type, WRL_String &data, WRL_Word &size);
    WRL_Byte GetParamCount();
    TW3RepActionParam * GetParam(WRL_DWord index) const;
    const WRL_String GetParamString() const;
};

typedef void (*TProcessActionCallBack)(TW3RepFile * m_owner, TW3RepAction * action, bool ifcount);

class TW3RepActionList
{
public:
    TW3RepActionList():m_loaded(false), m_owner(0), m_actions(0), m_alloccount(0), m_actioncount(0), m_blockcount(0), m_actiondetail(false), m_extraaction(false), m_gamebuild(0), m_ProcessActionCB(0) {}
    virtual ~TW3RepActionList();
    bool Load(TW3RepFile * owner, WRL_String &data, bool actiondetail, bool extraaction, WRL_Word gamebuild);
    void Unload();
    bool Loaded() const;
private:
    bool m_loaded;
    TW3RepFile * m_owner;
    TW3RepAction * m_actions;
    WRL_DWord m_alloccount;
    WRL_DWord m_actioncount;
    WRL_DWord m_blockcount;
    WRL_Word m_gamebuild;
    bool m_actiondetail;
    bool m_extraaction;
    TProcessActionCallBack m_ProcessActionCB;
    bool ProcessActionBlock(WRL_String &data, WRL_DWord &time, bool &paused, bool &started);
    void ProcessAction(WRL_String &data, WRL_Word &size, WRL_DWord time, bool &paused);
    TW3RepAction * AddAction(TW3RepActionID aid, WRL_Byte playerid, WRL_DWord time, TParamFunc paramfunc);
public:
    WRL_DWord GetActionCount() const;
    WRL_DWord GetBlockCount() const;
    TW3RepAction * GetAction(WRL_DWord index) const;
    void SetProcessActionCallBack(TProcessActionCallBack cb);
};

#endif

