#ifndef W3RepGameDataH
#define W3RepGameDataH

#include <stdio.h>
#include "W3RepDataType.h"

typedef struct {
    WRL_Byte magic[28];
    WRL_DWord header_size;
    WRL_DWord file_size;
    WRL_DWord header_version;
    WRL_DWord decomp_size;
    WRL_DWord block_num;
} TW3RepMainHeader;

typedef struct {
    WRL_Word version1;
    WRL_Word version2;
    WRL_Word buildnum;
    WRL_Word flag;
    WRL_DWord length;
    WRL_DWord crc32;
} TW3RepSubHeader0;

typedef struct {
    WRL_DWord clienttag;
    WRL_Word version2;
    WRL_Word version1;
    WRL_Word buildnum;
    WRL_Word flag;
    WRL_DWord length;
    WRL_DWord crc32;
} TW3RepSubHeader1;

class TW3RepHeader
{
public:
    TW3RepHeader();
    virtual ~TW3RepHeader(){}
    bool Load(FILE * fp);
    void Unload();
    bool Loaded() const;
private:
    bool m_loaded;
    WRL_DWord m_version;
public:
    TW3RepMainHeader Header;
    union {
        TW3RepSubHeader0 v0;
        TW3RepSubHeader1 v1;
    } SubHeader;
    const char *  GetVersionWRL_String(char * vstring, bool longversion) const;
    WRL_DWord GetVersion() const;
    WRL_Word GetBuild() const;
    WRL_DWord GetLength() const;
};

typedef enum {
    gametype_unknown = 0,
    gametype_ladder = 1,
    gametype_custom = 2,
    gametype_single = 3,
    gametype_team = 4
} TGameType;

enum {
    gameflag_hideterrain = 0x0001,
    gameflag_fullshare = 0x0002,
    gameflag_speedfast = 0x0004,
    gameflag_mapexplored = 0x0008,
    gameflag_alwaysvisible = 0x0010,
    gameflag_defaultvisible = 0x0020,
    gameflag_fullobserver = 0x0040,
    gameflag_observerondefeat = 0x0080,
    gameflag_teamtogether = 0x0100,
    gameflag_fixedteams = 0x0200,
    gameflag_randomhero = 0x0400,
    gameflag_randomraces = 0x0800,
    gameflag_referee = 0x1000,
    gameflag_fixedally = 0x2000,
    gameflag_fixedplayerprop = 0x4000,
    gameflag_fixracetorandom = 0x8000
};

class TW3RepGameData
{
public:
    TW3RepGameData();
    virtual ~TW3RepGameData();
    bool Load(WRL_String &data);
    void Unload();
    bool Loaded() const;
private:
    bool m_loaded;
    const char * m_gamename;
    WRL_Word m_gameflag;
    const char * m_mapname;
    const char * m_creator;
    WRL_Byte m_playercount;
    TGameType m_gametype;
    bool m_private;
    bool DecryptInfo(WRL_String &enc, const char * &mapname, const char * &creator) const;
public:
    const char * GetGameName() const;
    WRL_Word GetGameFlag() const;
    const char * GetMapName() const;
    const char * GetCreator() const;
    WRL_Byte GetPlayerCount() const;
    TGameType GetGameType() const;
    bool GetPrivate() const;
    bool SetGameName(const char * gamename);
    bool SetGameFlag(WRL_Word gameflag);
    bool SetMapName(const char * mapname);
    bool SetCreator(const char * creator);
    bool SetPlayerCount(WRL_Byte playercount);
    bool SetGameType(TGameType gametype);
    void SetPrivate(bool priv);
};

#endif