#include <stdlib.h>
#include <string.h>
#include "W3RepPlayer.h"

TW3RepPlayer::TW3RepPlayer()
{
    m_loaded = false;
    m_id = 0;
    m_name = NULL;
    m_slotstatus = slotstatus_open;
    m_computer = false;
    m_team = 0;
    m_color = 0;
    m_race = 0;
    m_ai = computerai_normal;
    m_handicap = 100;
    m_playtime = 0;
    m_actions = 0;
}

TW3RepPlayer::~TW3RepPlayer()
{
    Unload();
}

bool TW3RepPlayer::Load1(WRL_String &data, bool * ladder)
{
    bool extradata = false;

/*
    player status field 1 magic, 0x0 for host, 0x16 for others
*/
    if((*data) != 0x0 && (*data) != 0x16)
        return false;

    if(m_loaded)
        Unload();

    if((*data) == 0x16)
        extradata = true;

/*
    player id/name/race
*/
    data ++;
    m_id = (*data);

    data ++;
    m_name = strdup((const char *)data);
    data += (strlen(m_name)+1);
    if((*data) == 8)
    {
        m_race = GetRaceID(*(data + 5));
        if(ladder)
            (*ladder) = true;
    }
    else
        if(ladder)
            (*ladder) = false;

    data += ((*data) + 1);

    if(extradata) data += 4;

/*
    official replay(Tounament) used variables(won't access Load2() anymore)
*/
    m_slotstatus = slotstatus_used;
    m_team = (m_id - 1) % 2;

    m_loaded = true;
    return true;
}

bool TW3RepPlayer::Load2(WRL_String &data, WRL_DWord version)
{
    data += 2;
    m_slotstatus = (TSlotStatus)(*data);
    data ++;
    m_computer = ((*data) > 0);
    data ++;
    m_team = (*data);
    data ++;
    m_color = (*data);
    data ++;
    m_race = GetRaceID(*data);
    data ++;
    if(version > 2)
    {
        m_ai = (TComputerAI)(*data);
        data ++;
        if(version > 6)
        {
            m_handicap = (*data);
            data ++;
        }
    }
    m_loaded = true;
    return true;
}

void TW3RepPlayer::Unload()
{
    if(!m_loaded)
        return;
    m_loaded = false;
    m_id = 0;
    if(m_name)
    {
        free((void *)m_name);
        m_name = 0;
    }
    m_slotstatus = slotstatus_open;
    m_computer = false;
    m_team = 0;
    m_color = 0;
    m_race = 0;
    m_ai = computerai_normal;
    m_handicap = 100;
    m_playtime = 0;
    m_actions = 0;
}

bool TW3RepPlayer::Loaded() const
{
    return m_loaded;
}

WRL_Byte TW3RepPlayer::GetID() const
{
    return m_id;
}

const char * TW3RepPlayer::GetName() const
{
    return m_name;
}

TSlotStatus TW3RepPlayer::GetSlotStatus() const
{
    return m_slotstatus;
}

bool TW3RepPlayer::GetComputer() const
{
    return m_computer;
}

WRL_Byte TW3RepPlayer::GetTeam() const
{
    return m_team;
}

WRL_Byte TW3RepPlayer::GetColor() const
{
    return m_color;
}

WRL_Byte TW3RepPlayer::GetRace() const
{
    return m_race;
}

TComputerAI TW3RepPlayer::GetAI() const
{
    return m_ai;
}

WRL_Byte TW3RepPlayer::GetHandicap() const
{
    return m_handicap;
}

WRL_DWord TW3RepPlayer::GetPlayTime() const
{
    return m_playtime;
}

WRL_DWord TW3RepPlayer::GetActions() const
{
    return m_actions;
}

bool TW3RepPlayer::SetID(WRL_Byte id)
{
    if(id > 12)
        return false;
    m_id = id;
    return true;
}

bool TW3RepPlayer::SetName(const char * name)
{
    if(!name)
        return false;
    m_name = strdup(name);
    return true;
}

bool TW3RepPlayer::SetSlotStatus(TSlotStatus slotstatus)
{
    if(slotstatus > 2)
        return false;
    m_slotstatus = slotstatus;
    return true;
}

void TW3RepPlayer::SetComputer(bool computer)
{
    m_computer = computer;
}

bool TW3RepPlayer::SetTeam(WRL_Byte team)
{
    if(team > 12)
        return false;
    m_team = team;
    return true;
}

bool TW3RepPlayer::SetColor(WRL_Byte color)
{
    if(color > 12)
        return false;
    m_color =color;
    return true;
}

bool TW3RepPlayer::SetRace(WRL_Byte race)
{
    if((race > 5 && race < 8) || race >13)
        return false;
    m_race = race;
    return true;
}

bool TW3RepPlayer::SetAI(TComputerAI ai)
{
    if(ai > 2)
        return false;
    m_ai = ai;
    return true;
}

bool TW3RepPlayer::SetHandicap(WRL_Byte handicap)
{
    if(handicap != 50 && handicap != 60 && handicap != 70 && handicap != 80 && handicap != 90 && handicap != 100)
        return false;
    m_handicap = handicap;
    return true;
}

WRL_Byte TW3RepPlayer::GetRaceID(WRL_Byte raceid)
{
    switch(raceid & 0x3f)
    {
    case 0x01: return race_human;
    case 0x02: return race_orc;
    case 0x04: return race_nightelf;
    case 0x08: return race_undead;
    case 0x10: return race_demon;
    case 0x20: return race_random;
    default: return race_unknown;
    }
}

void TW3RepPlayer::SetPlayTime(WRL_DWord playtime)
{
    m_playtime = playtime;
}

void TW3RepPlayer::SetActions(WRL_DWord actions)
{
    m_actions = actions;
}

void TW3RepPlayer::IncrAction()
{
    m_actions ++;
}
