#include <stdlib.h>
#include <memory.h>
#include "zlib/zlib.h"
#include "ZUnpack.h"

typedef struct {
    WRL_Word complen, uncomplen;
    WRL_DWord checksum;
} TBlockHeader;

bool zunpack_data(FILE * fp, int blocknum, WRL_String &data, WRL_DWord & size)
{
    TBlockHeader h;
    if(fp == NULL)
        return false;
    size = 0;
    if(data != NULL) free(data);

    for(int i = 0; i < blocknum; i ++) {
        z_stream zc;
        WRL_String tempdata;
        if(!fread(&h, 8, 1, fp))
            return false;
        tempdata = (WRL_String )malloc(h.complen);
        if(!fread(tempdata, h.complen, 1, fp))
        {
            free((void *) tempdata);
            return false;
        }
        if(size == 0)
            data = (WRL_String )malloc(h.uncomplen);
        else
            data = (WRL_String )realloc(data, size + h.uncomplen);
        memset(&zc, 0, sizeof(z_stream));
        zc.next_in=tempdata;
        zc.avail_in=h.complen;
        zc.next_out=&data[size];
        zc.avail_out=h.uncomplen;
        if(inflateInit(&zc) != Z_OK)
        {
            free((void *) tempdata);
            return false;
        }
        if(inflate(&zc,Z_SYNC_FLUSH) != Z_OK)
        {
            free((void *) tempdata);
            return false;
        }
        if(inflateEnd(&zc) != Z_OK)
        {
            free((void *) tempdata);
            return false;
        }
        size += h.uncomplen;
        free((void *) tempdata);
    }

    return true;
}

